function Com = cooccurrence(img,max_levels,num_dist)
% COOCCURRENCE Computes co-occurrence matrices.
%
% Synopsis:
%  Com = cooccurrence(img,max_levels,num_dist)
%
% Description:
%  It computes co-occurrence matrices for image img
%  with at most max_levels gray-lavels.
%  The co-occurence matrices are computed for angles
%  0, 45, 90 and 135 and distances 1:num_dist.
%  
% Input:
%  img [h x w] Input image.
%  max_levels [1x1] Maximal number of gray-levels.
%  num_dist [1x1] Number of used distances.
%
% Output:
%  Com [max_levels x max_levels x 8] The co-occurence matrices:
%   Com(:,:,1) ... angle 0, distance 1
%   Com(:,:,2) ... angle 45, distance 1
%   Com(:,:,3) ... angle 90, distance 1
%   Com(:,:,4) ... angle 135, distance 1
%   Com(:,:,5) ... angle 0, distance 2
%   Com(:,:,6) ... angle 45, distance 2
%   ...
%  

% Modifications:
% 11-aug-2004, VF

[h,w] = size(img);

Com = zeros(max_levels,max_levels,4*num_dist);

for d=1:num_dist,
 for i=1:h,
  for j=1:w,

    Com(:,:,1+(d-1)*4) = update_com(Com(:,:,1+(d-1)*4),rel_W(img,d,i,j));
    Com(:,:,1+(d-1)*4) = update_com(Com(:,:,1+(d-1)*4),rel_E(img,d,i,j));

    Com(:,:,2+(d-1)*4) = update_com(Com(:,:,2+(d-1)*4),rel_EN(img,d,i,j));
    Com(:,:,2+(d-1)*4) = update_com(Com(:,:,2+(d-1)*4),rel_WS(img,d,i,j));

    Com(:,:,3+(d-1)*4) = update_com(Com(:,:,3+(d-1)*4),rel_N(img,d,i,j));
    Com(:,:,3+(d-1)*4) = update_com(Com(:,:,3+(d-1)*4),rel_S(img,d,i,j));
    
    Com(:,:,4+(d-1)*4) = update_com(Com(:,:,4+(d-1)*4),rel_WN(img,d,i,j));
    Com(:,:,4+(d-1)*4) = update_com(Com(:,:,4+(d-1)*4),rel_ES(img,d,i,j));
    
  end
 end
end

return;

%-------------------------------------------------------
function A = update_com(B,level)

A = B;

if ~isempty(level),
  a = level(1); b = level(2);
  A(a,b) = A(a,b) + 1;
end

%-------------------------------------------------------
function res = rel_E(img,d,i,j)
% angle 0, distance d

[h,w] = size(img);

if j+d <= w, 
  res(1) = img(i,j);
  res(2) = img(i,j+d);
else
  res = [];
end

%-------------------------------------------------------
function res = rel_W(img,d,i,j)
% angle 180, distance d

[h,w] = size(img);

if j > d, 
  res(1) = img(i,j);
  res(2) = img(i,j-d);
else
  res = [];
end

%-------------------------------------------------------
function res = rel_WN(img,d,i,j)
% angle 135, distance d

[h,w] = size(img);
if i > d & j > d,
  res(1) = img(i,j);
  res(2) = img(i-d,j-d);
else
  res = [];
end

%-------------------------------------------------------
function res = rel_ES(img,d,i,j)
% angle -45, distance d

[h,w] = size(img);
if i+d <= h & j+d <= w,
  res(1) = img(i,j);
  res(2) = img(i+d,j+d);
else
  res = [];
end

%-------------------------------------------------------
function res = rel_N(img,d,i,j)
% angle 90, distance d

[h,w] = size(img);
if i > d,
  res(1) = img(i,j);
  res(2) = img(i-d,j);
else
  res = [];
end
  
%-------------------------------------------------------
function res = rel_S(img,d,i,j)
% angle -90, distance d

[h,w] = size(img);
if i+d <= h,
  res(1) = img(i,j);
  res(2) = img(i+d,j);
else
  res = [];
end


%-------------------------------------------------------
function res = rel_EN(img,d,i,j)
% angle 45, distance d

[h,w] = size(img);
if i > d & j+d <= w,
  res(1) = img(i,j);
  res(2) = img(i-d,j+d);
else
  res=[];
end

%-------------------------------------------------------
function res = rel_WS(img,d,i,j)
% angle -135, distance d

[h,w] = size(img);
if i+d <= h & j > d,
  res(1) = img(i,j);
  res(2) = img(i+d,j-d);
else
  res = [];
end
